function [GaussNodes, GaussWeights, AntiGaussNodes, AntiGaussWeights] = GaussPlusAntiGauss(alpha, beta, method)
%
%   [GaussNodes, GaussWeights, AntiGaussNodes, AntiGaussWeights] = GaussPlusAntiGauss(alpha, beta, method)
%
%   Inputs: 
%       alpha - n+1 recurrence coefficients alpha_0,...,alpha_n, all real
%       beta - n+1 recurrence coefficients beta_0,...,beta_n, all real
%       method = optional argument either 'dandcPSF' or 'tqrPSF'
%
%   Outputs:
%       GaussNodes - the n Gauss nodes ordered from left to right
%       GaussWeights - the n Gauss weights
%       AntiGaussNodes - the n+1 Anti-Gauss nodes ordered from left to right
%       AntiGaussWeights - the n+1 Anti-Gauss weights
%
%   Computes the nodes and weights of an n-point Gauss rule for the weight
%   function whose associated monic orthogonal polynomial system has
%   recurrence coefficients given by alpha and beta. And then computes an
%   (n+1)-point Anti-Gauss rule for the same OPS using the method outlined
%   in [1]. Both alpha and beta must be column vectors, the elements of
%   beta must be non-negative, and beta_0 is assumed to be the zero'th
%   moment of the weight function. The size, n, of the rule will be one
%   less than the number of elements of alpha. The optional argument
%   specifies the method of constructing the rule which can be either the
%   Partial Spectral Factorization by Divide and Conquer method or the
%   Partial Spectral Factorization by TQR method. This is included for
%   comparative purposes and defaults to Divide and Conquer.
%
%   [1] H. Alqahtani, C.F. Borges, D.Lj. Djukić, R.M. Mutavdžić Djukić, L.
%   Reichel, M.M. Spalević, Computation of pairs of related Gauss-type
%   quadrature rules, Applied Numerical Mathematics, 2024
%
%   1 MARCH 2024 by Carlos F. Borges

% Argument validation
arguments
    alpha(:, 1) {mustBeReal}
    beta(:, 1) {mustBeNonnegative}
    method(1, :) char{mustBeMember(method, {'dandcPSF', 'tqrPSF'})} = 'dandcPSF'
end
if length(alpha) ~= length(beta)
    error('Inputs alpha and beta must be the same length.')
end

% Initialize.
beta_0 = beta(1);
 
% Construct the Gauss rule and capture the final row of the eigenvector
% matrix using the Partial Spectral Factorization.
if strcmp(method, 'dandcPSF')
    [GaussNodes,u_1,u_n] = dandcPSF(alpha(1:end-1), sqrt(beta(2:end-1)), false);
else
    [GaussNodes,u_1,u_n] = tqrPSF(alpha(1:end-1), sqrt(beta(2:end-1)), false);
end
GaussWeights = beta_0*u_1.*u_1;
 
% Construct the anti-Gauss rule.
[AntiGaussNodes,AntiGaussWeights] = eig_arrow(GaussNodes,sqrt(2*beta(end))*u_n,alpha(end),[u_1' 0]);
AntiGaussWeights = beta_0*(AntiGaussWeights.*AntiGaussWeights)';