function [GaussNodes, GaussWeights, AntiGaussNodes, AntiGaussWeights] = GaussPlusAntiGaussTraditional(alpha, beta)
%
%   [GaussNodes, GaussWeights, AntiGaussNodes, AntiGaussWeights] = GaussPlusAntiGaussTraditional(alpha, beta, method)
%
%   Inputs: 
%       alpha - n+1 recurrence coefficients alpha_0,...,alpha_n, all real
%       beta - n+1 recurrence coefficients beta_0,...,beta_n, all real
%
%   Outputs:
%       GaussNodes - the n Gauss nodes ordered from left to right
%       GaussWeights - the n Gauss weights
%       AntiGaussNodes - the n+1 Anti-Gauss nodes ordered from left to right
%       AntiGaussWeights - the n+1 Anti-Gauss weights
%
%   Computes the nodes and weights of an n-point Gauss rule for the weight
%   function whose associated monic orthogonal polynomial system has
%   recurrence coefficients given by alpha and beta using the TQR
%   Golub-Welsch approach. And then computes an (n+1)-point Anti-Gauss rule
%   for the same OPS also using the TQR Golub-Welsch approach. Both alpha
%   and beta must be column vectors, the elements of beta must be
%   non-negative, and beta_0 is assumed to be the zero'th moment of the
%   weight function. The size, n, of the rule will be one less than the
%   number of elements of alpha.  This code is used for performance testing
%   purposes only.
%
%   1 MARCH 2024 by Carlos F. Borges

% Argument validation
arguments
    alpha(:, 1) {mustBeReal}
    beta(:, 1) {mustBeNonnegative}
end
if length(alpha) ~= length(beta)
    error('Inputs alpha and beta must be the same length.')
end

% Initialize.
beta_0 = beta(1);
 
% Construct the Gauss rule and capture the final row of the eigenvector
% matrix using the TQR Golub-Welsch approach.
[GaussNodes,u_1] = tqrPSF(alpha(1:end-1), sqrt(beta(2:end-1)), false);
GaussWeights = beta_0*u_1.*u_1;
 
% Construct the anti-Gauss rule using the TQR Golub-Welsch approach.
beta(end) = 2*beta(end);
[AntiGaussNodes,AntiGaussWeights] = tqrPSF(alpha,beta(2:end));
AntiGaussWeights = beta_0*(AntiGaussWeights.*AntiGaussWeights)';