function [GaussNodes, GaussWeights, LobattoNodes, LobattoWeights] = GaussPlusGaussLobatto(alpha, beta, a, b, method)
%
%   [GaussNodes, GaussWeights, LobattoNodes, LobattoWeights] = GaussPlusGaussLobatto(alpha, beta, a, b, method)
%
%   Inputs: 
%       alpha - n+1 recurrence coefficients alpha_0,...,alpha_n, all real
%       beta - n+1 recurrence coefficients beta_0,...,beta_n, all real
%       method = optional argument either 'dandcPSF' or 'tqrPSF'
%
%   Outputs:
%       GaussNodes - the n Gauss nodes ordered from left to right
%       GaussWeights - the n Gauss weights
%       LobattoNodes - the n+1 Gauss-Lobatto nodes ordered from left to right
%       LobattoWeights - the n+1 Gauss-Lobatto weights
%
%   Computes the nodes and weights of an n-point Gauss rule for the weight
%   function whose associated monic orthogonal polynomial system has
%   recurrence coefficients given by alpha and beta. And then computes an
%   n+1 point Gauss-Lobatto rule (using the method oulined in [1] and a
%   construction from [2]) with specified nodes a and b for the same OPS.
%   Both alpha and beta must be column vectors, the elements of beta must
%   be non-negative, and beta_0 is assumed to be the zero'th moment of the
%   weight function. The size, n, of the rule will be the same as the
%   number of elements of alpha. The optional argument specifies the method
%   of constructing the rule which can be either the Partial Spectral
%   Factorization by Divide and Conquer method or the Partial Spectral
%   Factorization by TQR method. This is included for comparative purposes
%   and defaults to Divide and Conquer.
%
%   Note: The code is agnostic as to the location of the preassigned nodes
%   a and b. Classically, these nodes are placed at the left and right
%   endpoints of the interval of integration. However, the code will
%   attempt to place them at any specified points provided the modified
%   matrix eigenvalue problem is well-defined. If not it throws and error.
%
%   [1] H. Alqahtani, C.F. Borges, D.Lj. Djukić, R.M. Mutavdžić Djukić, L.
%   Reichel, M.M. Spalević, Computation of pairs of related Gauss-type
%   quadrature rules, Applied Numerical Mathematics, 2024
%
%   [2] Golub, G.H. "Some modified matrix eigenvalue problems", SIAM Review,
%   1973.
%
%   1 MARCH 2024 by Carlos F. Borges

% Argument validation
arguments
    alpha(:, 1) {mustBeReal}
    beta(:, 1) {mustBeNonnegative}
    a,b(1,1) {mustBeReal}
    method(1, :) char{mustBeMember(method, {'dandcPSF', 'tqrPSF'})} = 'dandcPSF'
end
if length(alpha) ~= length(beta)
    error('Inputs alpha and beta must be the same length.')
end


% Initialize.
beta_0 = beta(1);

% Construct the Gauss rule and capture the final row of the eigenvector
% matrix using the Partial Spectral Factorization.
if strcmp(method, 'dandcPSF')
    [GaussNodes,u_1,u_n] = dandcPSF(alpha, sqrt(beta(2:end)), false);
else
    [GaussNodes,u_1,u_n] = tqrPSF(alpha, sqrt(beta(2:end)), false);
end
GaussWeights = beta_0*u_1.*u_1;

% Compute the gamma_n and mu_n values by solving the equations 7.9 from
% [1] using knowledge of the spectral factorization of the nxn Jacobi
% matrix from above. 
gamma_n = u_n'*(u_n./(GaussNodes-a));
mu_n = u_n'*(u_n./(GaussNodes-b));
% Solving equations 7.11 from [1] then yields the extension to the
% original Jacobi matrix. However, this can fail for inappropriate choices
% of the a and b nodes.
beta_n_squared = (a-b)/(mu_n-gamma_n);
if ~isfinite(beta_n_squared) || (beta_n_squared < 0)
     error('Gauss-Lobatto rule does not exist for the given points a and b.')
end
alpha_np1 = a + beta_n_squared*gamma_n;

% Compute the Gauss-Lobatto nodes and weights by finding the eigenvalues
% and the first row of the normalized eigenvectors of an arrow matrix.
[LobattoNodes,LobattoWeights] = eig_arrow(GaussNodes,sqrt(beta_n_squared)*u_n,alpha_np1,[u_1' ,0]);
LobattoWeights = beta_0*(LobattoWeights.*LobattoWeights)';

end

