function [GaussNodes, GaussWeights, RadauNodes, RadauWeights] = GaussPlusGaussRadau(alpha, beta, a, method)
%
%   [GaussNodes, GaussWeights, RadauNodes, RadauWeights] = GaussPlusGaussRadau(alpha, beta, a, method)
%
%   Inputs: 
%       alpha - n+1 recurrence coefficients alpha_0,...,alpha_n, all real
%       beta - n+1 recurrence coefficients beta_0,...,beta_n, all real
%       method = optional argument either 'dandcPSF' or 'tqrPSF'
%
%   Outputs:
%       GaussNodes - the n Gauss nodes ordered from left to right
%       GaussWeights - the n Gauss weights
%       RadauNodes - the n+1 Gauss-Radau nodes ordered from left to right
%       RadauWeights - the n+1 Gauss-Radau weights
%
%   Computes the nodes and weights of an n-point Gauss rule for the weight
%   function whose associated monic orthogonal polynomial system has
%   recurrence coefficients given by alpha and beta. And then computes an
%   n+1 point Gauss-Radua rule (using the method oulined in [1] and a
%   construction from [2]) with specified node a for the same OPS. Both
%   alpha and beta must be column vectors, the elements of beta must be
%   non-negative, and beta_0 is assumed to be the zero'th moment of the
%   weight function. The size, n, of the rule will be one less than the
%   number of elements of alpha. The optional argument specifies the method
%   of constructing the rule which can be either the Partial Spectral
%   Factorization by Divide and Conquer method or the Partial Spectral
%   Factorization by TQR method. This is included for comparative purposes
%   and defaults to Divide and Conquer.
%
%   Note: The code is agnostic as to the location of the preassigned node
%   a. Classically, this node is placed at the left endpoint of the
%   interval of integration. However, the code will attempt to place it at
%   any specified point provided the modified matrix eigenvalue problem is
%   well-defined. If not it throws and error.
%
%   [1] H. Alqahtani, C.F. Borges, D.Lj. Djukić, R.M. Mutavdžić Djukić, L.
%   Reichel, M.M. Spalević, Computation of pairs of related Gauss-type
%   quadrature rules, Applied Numerical Mathematics, 2024
%
%   [2] Golub, G.H. "Some modified matrix eigenvalue problems", SIAM Review,
%   1973.
%
%   1 MARCH 2024 by Carlos F. Borges

% Argument validation
arguments
    alpha(:, 1) {mustBeReal}
    beta(:, 1) {mustBeNonnegative}
    a(1,1) {mustBeReal}
    method(1, :) char{mustBeMember(method, {'dandcPSF', 'tqrPSF'})} = 'dandcPSF'
end
if length(alpha) ~= length(beta)
    error('Inputs alpha and beta must be the same length.')
end


% Initialize.
beta_0 = beta(1);
beta_n = beta(end);

% Construct the Gauss rule and capture the final row of the eigenvector
% matrix using the Partial Spectral Factorization.
if strcmp(method, 'dandcPSF')
    [GaussNodes,u_1,u_n] = dandcPSF(alpha(1:end-1), sqrt(beta(2:end-1)),false);
else
    [GaussNodes,u_1,u_n] = tqrPSF(alpha(1:end-1), sqrt(beta(2:end-1)),false);
end
GaussWeights = beta_0*u_1.*u_1;

% Compute delta_n by solving the equation 7.5 from [1] using knowledge of
% the spectral factorization of the nxn Jacobi matrix from above. Note
% that this can fail for inappropriate choices of a.
delta_n = beta_n*(u_n'*(u_n./(GaussNodes-a)));
if ~isfinite(delta_n)
     error('The Gauss-Radau rule does not exist for the given value of a.');
end
% Solving equation 7.6 from [1] then yields the modified value of
% alpha_{n+1}. 
alpha_np1 = a + delta_n;

% Compute the Gauss-Radau nodes and weights by finding the eigenvalues
% and the first row of the normalized eigenvectors of an arrow matrix.
[RadauNodes,RadauWeights] = eig_arrow(GaussNodes,sqrt(beta_n)*u_n,alpha_np1,[u_1' 0]);
RadauWeights = beta_0*(RadauWeights.*RadauWeights)';

end

