function [GaussNodes, GaussWeights, OAGNodes, OAGWeights, sigma] = GaussPlusOptimalAveragedGauss(alpha, beta, method)
%
%   [GaussNodes, GaussWeights, OAGNodes, OAGWeights, sigma] = GaussPlusOptimalAveragedGauss(alpha, beta, method)
%
%   Inputs: 
%       alpha - n+2 recurrence coefficients alpha_0,...,alpha_{n+1}, all real
%       beta - n+2 recurrence coefficients beta_0,...,beta_{n+1}, all real
%       method = optional argument either 'dandcPSF' or 'tqrPSF'
%
%   Outputs:
%       GaussNodes - the n Gauss nodes ordered from left to right
%       GaussWeights - the n Gauss weights
%       OAGNodes - the n+1 added Optimal Averaged Gauss nodes ordered from left to right
%       OAGWeights - the n+1 added Optimal Averaged Gauss weights
%       sigma - weighting coefficient for combining the rules
%
%   Computes the nodes and weights of an n-point Gauss rule for the weight
%   function whose associated monic orthogonal polynomial system has
%   recurrence coefficients given by alpha and beta. And then uses the
%   method outlined in [1] to compute n+1 additional nodes and weights that
%   can be combined with the n-point Gauss rule to construct a (2n+1)-point
%   optimal averaged Gauss rule. The output value sigma is the weighting
%   coefficient that allows one to combine the two parts. The full
%   (2n+1)-point optimal averaged Gauss rule can be constructed by merging
%   the GaussNodes with the OAGNodes and the sigma*GaussWeights merged with
%   (1-sigma)*OAGWeights. Both alpha and beta must be column vectors, the
%   elements of beta must be non-negative, and beta_0 is assumed to be the
%   zero'th moment of the weight function. The size, n, of the rule will be
%   two less than the number of elements of alpha. The optional argument
%   specifies the method of constructing the rule which can be either the
%   Partial Spectral Factorization by Divide and Conquer method or the
%   Partial Spectral Factorization by TQR method. This is included for
%   comparative purposes and defaults to Divide and Conquer.
%
%   [1] H. Alqahtani, C.F. Borges, D.Lj. Djukić, R.M. Mutavdžić Djukić, L.
%   Reichel, M.M. Spalević, Computation of pairs of related Gauss-type
%   quadrature rules, Applied Numerical Mathematics, 2024
%
%   1 MARCH 2024 by Carlos F. Borges

% Argument validation
arguments
    alpha(:, 1) {mustBeReal}
    beta(:, 1) {mustBeNonnegative}
    method(1, :) char{mustBeMember(method, {'dandcPSF', 'tqrPSF'})} = 'dandcPSF'
end
if length(alpha) ~= length(beta)
    error('Inputs alpha and beta must be the same length.')
end

% Initialize.
alpha_n = alpha(end-1);
beta_0 = beta(1);
beta_tilde = beta(end-1)+beta(end);
sigma = (beta(end)/beta_tilde);

% Construct the Gauss rule and capture the final row of the eigenvector
% matrix using the Partial Spectral Factorization.
if strcmp(method, 'dandcPSF')
    [GaussNodes,u_1,u_n] = dandcPSF(alpha(1:end-2), sqrt(beta(2:end-2)), false);
else
    [GaussNodes,u_1,u_n] = tqrPSF(alpha(1:end-2), sqrt(beta(2:end-2)), false);
end
GaussWeights = beta_0*u_1.*u_1;


% Compute the nodes and weights of the rest of the optimal averaged Gauss
% rule.
[OAGNodes,OAGWeights] = eig_arrow(GaussNodes,sqrt(beta_tilde)*u_n,alpha_n,[u_1' 0]);
OAGWeights = beta_0*(OAGWeights.*OAGWeights)';

end

