function [GaussNodes, GaussWeights, OAGNodes, OAGWeights] = GaussPlusOptimalAveragedGaussTraditional(alpha, beta)
%
%   [GaussNodes, GaussWeights, OAGNodes, OAGWeights, sigma] = GaussPlusOptimalAveragedGaussTraditional(alpha, beta, method)
%
%   Inputs: 
%       alpha - n+2 recurrence coefficients alpha_0,...,alpha_{n+1}, all real
%       beta - n+2 recurrence coefficients beta_0,...,beta_{n+1}, all real
%
%   Outputs:
%       GaussNodes - the n Gauss nodes ordered from left to right
%       GaussWeights - the n Gauss weights
%       OAGNodes - the n+1 added Optimal Averaged Gauss nodes ordered from left to right
%       OAGWeights - the n+1 added Optimal Averaged Gauss weights
%
%   Computes the nodes and weights of an n-point Gauss rule for the weight
%   function whose associated monic orthogonal polynomial system has
%   recurrence coefficients given by alpha and beta. And then constructs a
%   (2n+1)-point optimal averaged Gauss rule. This code is used for
%   performance testing purposes only.
%
%   1 MARCH 2024 by Carlos F. Borges

% Argument validation
arguments
    alpha(:, 1) {mustBeReal}
    beta(:, 1) {mustBeNonnegative}
end
if length(alpha) ~= length(beta)
    error('Inputs alpha and beta must be the same length.')
end

% Initialize.
beta_0 = beta(1);

% Construct the Gauss rule using the Partial Spectral Factorization.
[GaussNodes,u_1,~] = tqrPSF(alpha(1:end-2), sqrt(beta(2:end-2)), false);
GaussWeights = beta_0*u_1.*u_1;

% Now generate the Optimal Averaged Gauss rule.
n = length(alpha)-2;
beta_0 = beta(1); beta = beta(2:end); alpha = [alpha(1:n); alpha(n+1); flipud(alpha(1:n))]; beta = [beta; flipud(beta(1:n-1))];
[OAGNodes,OAGWeights] = tqrPSF(alpha, beta);
OAGWeights = beta_0*OAGWeights.*OAGWeights;

end

