function [nodes, weights] = GaussRule(alpha, beta, method)
%
%   [nodes,weights] = GaussRule(alpha,beta, method)
%
%   Inputs: 
%       alpha - n recurrence coefficients alpha_0,...,alpha_{n-1}, all real
%       beta - n recurrence coefficients beta_0,...,beta_{n-1}, all real
%       method = optional argument either 'dandcPSF' or 'tqrGW'
%
%   Outputs:
%       nodes - the Gauss nodes ordered from left to right
%       weights - the Gauss weights
%
%   Computes the nodes and weights of an n-point Gauss rule for the weight
%   function whose associated monic orthogonal polynomial system has
%   recurrence coefficients given by alpha and beta. Both alpha and beta
%   must be column vectors, the elements of beta must be non-negative, and
%   beta_0 is assumed to be the zero'th moment of the weight function. The
%   size, n, of the rule will be equal to the number of elements of alpha.
%   The optional argument specifies the method of constructing the rule
%   which can be either the Partial Spectral Factorization by Divide and
%   Conquer method or the Golub-Welsch TQR method. This is included for
%   comparative purposes and defaults to Divide and Conquer.
%
%   1 MARCH 2024 by Carlos F. Borges

% Argument validation
arguments
    alpha(:, 1) {mustBeReal}
    beta(:, 1) {mustBeNonnegative}
    method(1, :) char{mustBeMember(method, {'dandcPSF', 'tqrGW'})} = 'dandcPSF'
end
if length(alpha) ~= length(beta)
    error('Inputs alpha and beta must be the same length.')
end

% Compute the Gauss rule using the selected method.
beta_0 = beta(1);
if strcmp(method, 'dandcPSF')
    [nodes, weights, ~] = dandcPSF(alpha, sqrt(beta(2:end)));
else
    [nodes, weights] = tqrPSF(alpha, sqrt(beta(2:end)));
end
weights = beta_0 * weights .* weights;
end