function [alpha,beta] = JacobiRecurrence(n, a, b)
%
%   [alpha,beta] = JacobiRecurrence(n, a, b)
%   R = JacobiRecurrence(n, a, b)
% 
%   Computes the first n recurrence coefficients for the monic Jacobi
%   polynomials. These polynomials are orthogonal with respect to the
%   weight function w(x) = (1-x)^a*(1+x)^b on the interval [-1,1]. The code
%   is adapted from the chebfun jacpts() code. If there is only one output
%   argument then the recurrence coefficients are collected together in an
%   nx2 matrix.
%
%   1 MARCH 2024 by Carlos F. Borges

% Argument validation
arguments
    n (1,1) {mustBeInteger(n), mustBeGreaterThanOrEqual(n,1)}
    a (1,1) {mustBeGreaterThan(a,-1)}
    b (1,1) {mustBeGreaterThan(b,-1)}
end

% Compute the Jacobi polynomial recurrence coefficients. 
a_plus_b = a + b;
b_minus_a = b - a;
square_diff = a_plus_b*b_minus_a;
kk = (2:n-1)';
twokplus = 2*kk + a_plus_b;
alpha = [b_minus_a/(2 + a_plus_b)
        square_diff./((twokplus - 2).*twokplus)
        square_diff./((2*n - 2+a_plus_b).*(2*n+a_plus_b))];
beta = [2^(a_plus_b+1)*exp(betaln(a+1, b+1))
        4*( (1 + a)*(1 + b)/(a_plus_b + 3))/(a_plus_b + 2).^2
        4*(kk.*(kk + a).*(kk + b).*(kk + a_plus_b)./(twokplus.^2 - 1))./twokplus.^2];

if nargout < 2
    alpha = [alpha beta];
end