function [lambda, u_1, u_n] = dandcPSF(alpha, beta, firsts_positive)
%
%   [lambda, u_1, u_n] = dandcPSF(alpha, beta, firsts_positive)
%
%   Inputs: 
%       alpha - n diagonal elements, all real
%       beta - n-1 sub(sup)-diagonal elements, all real
%       firsts_positive - optional argument that controls the signs of u_k
%
%   Outputs:
%       lam - eigenvalues ordered from left to right
%       u_1 - first elements of the normalized eigenvectors
%       u_n - last elements of the normalized eigenvectors
%
%   Solves the symmetric tridiagonal eigenproblem using the divide and
%   conquer by extension algorithm as described in [1]. This is a recursive
%   implementation (one based on tree traversal would likely be faster but
%   less transparent.) The computed eigenvalues (lambda) are real and the
%   output is sorted so that the eigenvalues are increasing. The output
%   column vectors u_1 and u_n contain the elements of the first and last
%   rows, respectively, of the matrix of normalized eigenvectors. The
%   logical argument firsts_positive is optional and defaults to true if
%   not supplied by the user. If true the signs of the elements of u_1 and
%   u_n are permuted so that all of the elements of u_1 are positive, if
%   false then the elements of u_n will all be positive. The input alpha
%   must have exactly on more element than beta.
%
%   [1] C. F. Borges and W. B. Gragg, A parallel divide and conquer
%   algorithm for the generalized real symmetric definite tridiagonal
%   eigenproblem, in Numerical Linear Algebra, L. Reichel, A. Ruttan, and
%   R. S. Varga, eds., De Gruyter, 2011, pp. 11–30.
%
%   26 JANUARY 2024 by Carlos F. Borges

% Argument validation
arguments
    alpha(:, 1) {mustBeReal}
    beta(:, 1) {mustBeReal}
    firsts_positive logical = true
end
if length(alpha) ~= (length(beta)+1)
    error('Inputs alpha and beta are incompatibly sized.')
end

% It will be cleaner to proceed if we take all of the betas to be
% non-negative. This is easy to arrange for this calculation as we want
% only the eigenvalues and the first and last rows of the eigenvector
% matrix. The eigenvalues clearly do not depend on the signs of the betas
% but the sign relationship between u_1 and u_n will change if there are an
% odd number of negative betas. So we set flip_sign to true in that case
% so that it can be corrected later on and immediately replace beta with
% abs(beta).
flip_sign = (mod(sum(beta<0),2) > 0);
beta = abs(beta);


% Solve the problem by calling the interior function.
[lambda, u_1, u_n] = dandcPSF_interior(alpha, beta, firsts_positive);

% Fix the sign disagreement if flip_sign
if flip_sign
    if firsts_positive
        u_n = -u_n;
    else
        u_1 = - u_1;
    end
end

% Allow the user to collect a consolidated output.
if nargout < 2
    lambda = [lambda u_1 u_n];
end

end

% SUBFUNCTIONS

% This subfunction does the actual work but avoids repetitive argument
% validation that would result because of the use of recursion. The betas
% must be non-negative.
function [lambda, u_1, u_n] = dandcPSF_interior(alpha, beta, firsts_positive)

% Initialize
n = length(alpha);

% Handle the exceptional cases
if (n < 3) 
    switch n
        case 1
            lambda = alpha; 
            u_1 = 1; 
            u_n = 1;
        case 2
            [lambda, V] = solve2x2(alpha(1), beta, alpha(2), firsts_positive);
            u_1 = V(1,:)';
            u_n = V(2,:)';
        otherwise
            [lambda, u_1, u_n] = tqrPSF(alpha, beta, firsts_positive);
    end
    return;
else % Divide and Conquer

    % Divide phase
    m = floor(n/2); % Select the split point.

    % Solve the subproblems. Note that since we want the beta > 0 we need
    % the signs of the v1 elements to be positive. These two could be run
    % as a parallel block.
    [L1, u1, v1] = dandcPSF_interior(alpha(1:m), beta(1:m-1),false);
    [L2, u2, v2] = dandcPSF_interior(alpha(m+2:n), beta(m+2:n-1),true);

    % Combine phase.
    b1 = beta(m) * v1;
    b2 = beta(m+1) * u2;

    % A single step of merge sort puts the elements of the arrow in order.
    [a, p] = merge_sorted_ascending(L1, L2);
    b = [b1; b2];
    u_1 = [u1; zeros(n-m-1, 1)];
    u_n = [zeros(m, 1); v2];
    b = b(p);
    u_1 = u_1(p);
    u_n = u_n(p);

    % Find the eigenvalues of the arrow and the first and last rows of the
    % normalized eigenvectors.
    [lambda, V] = eig_arrow(a, b, alpha(m+1),[u_1', 0; u_n', 0]);

end

% Grab the first and last rows of the matrix of normalized eigenvectors as
% columns.
u_1 = V(1, :)';
u_n = V(end, :)';
% Correct the signs.
if firsts_positive
    tmp = (u_1<0);
    u_n(tmp) = -u_n(tmp);
    u_1 = abs(u_1);
else
    tmp = (u_n<0);
    u_1(tmp) = -u_1(tmp);
    u_n = abs(u_n);
end

end 
