function [lambda, V] = eig_arrow(alpha, beta, gamma, U)
%
%   [lambda, V] = eig_arrow(alpha, beta, gamma, U)
%
%   Inputs: 
%       alpha - n-1 shaft elements, all real and ordered left to right.
%       beta - n-1 barb elements, all real and non-negative.
%       gamma - the tip of the arrow, real.
%       U - an optional mxn matrix.
%
%   Outputs:
%       lam - eigenvalues ordered from left to right.
%       V - either the matrix of normalized eigenvectors or U multiplied on
%       the right by the matrix of normalized eigenvectors.
%
%   This function deflates the arrow, then calls the code for an unreduced
%   arrow to complete the computation of the eigenvalues and normalized
%   eigenvectors of the arrow, and finally puts the pieces back together.
%   It deals with combo deflations and beta deflations. It requires the
%   shaft, barb, and tip elements as input. If the user inputs a fourth
%   argument U then instead of returning the matrix of normalized
%   eigenvectors V, it returns U*V. Deflation is based on the analysis in
%   [1].
%
%   [1] C. F. Borges and W. B. Gragg, A parallel divide and conquer
%   algorithm for the generalized real symmetric definite tridiagonal
%   eigenproblem, in Numerical Linear Algebra, L. Reichel, A. Ruttan, and
%   R. S. Varga, eds., De Gruyter, 2011, pp. 11–30.
%
%   WARNING: For reasons of efficiency, this code has NO argument
%   validation and assumes WITHOUT CHECKING that the alpha are sorted from
%   left to right and that all of the beta >= 0.
%
%   30 SEPTEMBER 2023 by Carlos F. Borges

% Initialize
n = length(alpha)+1;
C = eye(n);


% This is the deflation tolerance from [1]
def_tol = 1.5*(n+6)*eps*norm(beta);

% Combo deflation This is a research code and so this is done rather
% inefficiently. It should be done using the same approach for accumulating
% and then applying plane rotations as in LAPACK (dlasr() and related
% functions.)
combo_deflation = false;
for k=1:n-2
    gap = alpha(k+1) - alpha(k);
    if gap < def_tol
        combo_deflation = true;
        %Construct the rotation.
        h = hypot(beta(k),beta(k+1));
        c = beta(k+1)/h;
        s = -beta(k)/h;
        R = [c -s;s c];
        C(:,k:k+1) = C(:,k:k+1)*R;
        beta(k) = 0;
        beta(k+1) = h;
    end
end
% End Combo deflation

% Beta deflation 
deflation = false;
deflation_indices = (abs(beta)<=def_tol);
Num_deflations = sum(deflation_indices);
if Num_deflations > 0
    deflation = true;
    deflation_eigenvalues = alpha(deflation_indices);
    deflation_eigenvectors = zeros(n,Num_deflations);
    deflation_eigenvectors(deflation_indices,:) = eye(Num_deflations);
    beta = beta(~deflation_indices);
    alpha = alpha(~deflation_indices);
end
% End Beta deflation

% Solve the deflated eigenvalue problem with a fully deflated arrow solver.
[lambda,V] = eig_arrow_deflated(alpha, beta, gamma);

% If there were deflations then we need to combine the two sets.
if deflation
    % Elevate the eigenvectors from the deflated problem to the dimension of the original problem
    tmp = zeros(n,length(lambda));
    tmp([~deflation_indices;true],:) = V;
    
    % Merge the two answers with a single step of merge sort.
    [lambda, p] = merge_sorted_ascending(deflation_eigenvalues,lambda);
    V = [deflation_eigenvectors tmp]; V = V(:,p);

    % Finally,if there was a combo deflation apply the rotation (again, the
    % LAPACK approach should be used here.)
    if combo_deflation
        V = C*V;
    end
end

if nargin > 3
    V = U*V;
end

end


