function [lambda, V] = eig_arrow_deflated(alpha, beta, gamma)
%
%   [lambda, V] = eig_arrow_deflated(alpha, beta, gamma)
%
%   Inputs: 
%       alpha - n-1 shaft elements, all real and ordered left to right.
%       beta - n-1 barb elements, all real and non-negative.
%       gamma - the tip of the arrow, real.
%
%   Outputs:
%       lam - eigenvalues ordered from left to right.
%       V - the matrix of normalized eigenvectors.
%
%   This function uses zero-finder described in [1] to find the eigenvalues
%   of the fully deflated arrow. Then it carefully computes the normalized
%   eigenvectors to avoid instability.  This code uses shifts so that zeros
%   are computed as corrections to the interval endpoint closest to the
%   eigenvalue. It requires the shaft, barb, and tip elements as input.
%
%   [1] C. F. Borges and W. B. Gragg, A parallel divide and conquer
%   algorithm for the generalized real symmetric definite tridiagonal
%   eigenproblem, in Numerical Linear Algebra, L. Reichel, A. Ruttan, and
%   R. S. Varga, eds., De Gruyter, 2011, pp. 11–30.
%
%   WARNING: For reasons of efficiency, this code has NO argument validation
%   and assumes WITHOUT CHECKING that the alpha are sorted from left to
%   right and that all of the beta > 0.
%
%   30 SEPTEMBER 2023 by Carlos F. Borges

% Initialize
MAX_ITER = 10;
n = length(alpha);

% Handle the exceptional cases
if n < 2
    switch n
        case 0
            lambda = gamma;
            V = 1;
            return;
        case 1
            [lambda, V] = solve2x2(alpha, beta, gamma);
            return;
    end
end

% We compute some constant quantities that are used repeatedly but do not
% depend on the shifts.
beta_sq = beta .* beta;
alpha_minus_alpha_transpose = alpha - alpha';
aq = alpha_minus_alpha_transpose(:, 1:end-1) .* alpha_minus_alpha_transpose(:, 2:end);

% We begin by determining the shift to be used for each of the eigenvalues.
% For the extreme eigenvalues the shifts are alpha(1) and alpha(n)
% respectively. For the interior eigenvalues we chose the alpha(k) or
% alpha(k+1) that is closest to the eigenvalue in that interval. This is
% decided by checking the sign of secular function evaluated at the
% midpoint. If it is positive then the eigenvalue is closer to the left
% endpoint, etc. Moreover, it is convenient at this time to determine the
% initial guess for each of the interior intervals. If the eigenvalue is in
% the left half of the interval we use one quarter of the width of the
% interval, if on the right we use negative one quarter of the width of the
% interval.

% We compute the interior shifts in a vectorized manner. We begin by
% setting the shifts to the right side of each interval and the initial
% guesses to negative one quarter of the width of the interval.
shifts = alpha(2:end);
xjs = -.25 * diff(alpha);
% We evaluate the secular function at the midpoints of each interval and
% find the logical indices of the intervals where the midpoint value is
% positive.
midpoints = .5 * (alpha(1:end-1) + alpha(2:end));
positive_fx = sum([beta_sq ./ (alpha - midpoints'); (midpoints' - gamma)]) > 0;
% Each of these shifts needs to be switched to the left endpoint and the
% corresponding initial guess needs to have its sign swapped.
shifts(positive_fx) = alpha(positive_fx);
xjs(positive_fx) = -xjs(positive_fx);
% And then we pad these arrays for the extremal eigenvalues.
shifts = [alpha(1); shifts; alpha(end)];
xjs = [0; xjs; 0];

% Conceptually, it is easiest to think of the problem as one of finding the
% zeros of a sequence of shifted secular functions. We represent the
% shifted functions as a matrix and vector.
a_bar = alpha - shifts';
g_bar = gamma - shifts';

% Begin by finding the eigenvalues.
% Run the Borges-Gragg iteration to find the interior eigenvalues.
for k = 2:n

    xjs(k) = find_interior(k);

end

% Run the find_right iteration on the negated shifted arrow to find the
% leftmost eigenvalue.
xjs(1) = -find_right(alpha(1)-alpha, alpha(1)-gamma);
shifts(1) = alpha(1);

% Run the find_right iteration on the shifted arrow to find the rightmost
% eigenvalue.
xjs(end) = find_right(alpha-alpha(end), gamma-alpha(end));
shifts(end) = alpha(end);

% The computed eigenvalues are:
lambda = shifts + xjs;

% Compute the eigenvectors.
alpha_minus_lambda = a_bar - xjs';

% To prevent a loss of orthogonality that causes instability we need to
% solve an inverse eigenvalue problem for a corrected set of betas.
beta = sqrt(-(prod(alpha_minus_lambda./[alpha_minus_alpha_transpose + eye(n), ones(n, 1)], 2)));

% Finally we compute the normalized eigenvectors.
V = [-beta ./ alpha_minus_lambda; ones(1, n+1)];
V = normc(V);

% INTERIOR SUBFUNCTIONS

    function [xj] = find_interior(k)
        % Uses Borges-Gragg iteration to find the interior eigenvalue
        % between alpha(k-1) and alpha(k) of a fully unreduced arrow.

        % Grab the starting guess.
        xj = xjs(k);

        % Do the Borges-Gragg iteration.
        for j = 1:MAX_ITER

            % Compute f(x)
            a_bar_minus_xj = a_bar(:, k) - xj;
            f_entire = xj - g_bar(k);
            f_rational = beta_sq ./ a_bar_minus_xj;
            fx = f_entire + sum(f_rational);

            % Check for termination.
            end_tol = 2 * eps * (2 * (abs(f_entire) + (n + 6) * sum(abs(f_rational))));
            if abs(fx) < end_tol
                break;
            end

            % Compute the step.
            tmp1 = f_rational ./ a_bar_minus_xj;
            fpx = 1 + sum(tmp1);
            tmp2 = sum(tmp1.*(aq(:, k-1) ./ a_bar_minus_xj));
            sigma = 2 * xj + f_entire - (a_bar(k-1, k) + a_bar(k, k)) + tmp2;
            aa = -sigma / (a_bar_minus_xj(k-1) * a_bar_minus_xj(k));
            bb = fpx - fx * (1 / a_bar_minus_xj(k-1) + 1 / a_bar_minus_xj(k));
            delta = 2 * fx / bb;
            delta = delta / (1 + sqrt(1+(2 * aa / bb)*delta));

            % Take the step but make sure we stay in the region.
            xj_notional = xj - delta;
            if xj_notional * xj > 0
                xj = xj_notional;
            else
                xj = .25 * xj;
            end

        end

    end

    function [xj] = find_right(a_bar, g_bar)
        % Uses Borges-Gragg iteration to find the rightmost eigenvalue of a fully
        % unreduced arrow whose rightmost alpha is at the origin.

        % The initial guess is the first iterate from infinity (Borges-Gragg p. 9).
        if g_bar > 0
            xj = .5 * g_bar + sqrt(.25*g_bar^2+sum(beta_sq));
        else
            xj = -sum(beta_sq) / (.5 * g_bar - sqrt(.25*g_bar^2+sum(beta_sq)));
        end

        for j = 1:MAX_ITER

            % Compute f(x)
            a_bar_minus_xj = a_bar - xj;
            f_entire = xj - g_bar;
            f_rational = beta_sq ./ a_bar_minus_xj;
            fx = f_entire + sum(f_rational);

            % Check for termination.
            end_tol = 2 * eps * (2 * (abs(f_entire) + (n + 6) * sum(abs(f_rational))));
            if abs(fx) < end_tol
                break;
            end

            % Compute the step.
            tmp1 = f_rational ./ a_bar_minus_xj;
            fpx = 1 + sum(tmp1);
            half_fppx = sum(tmp1./a_bar_minus_xj);
            a = fpx + half_fppx * xj;
            b = -(fx + fpx * xj);
            c = fx * xj;
            discriminant = b * b - 4 * a * c;
            if b > 0
                delta = (-b - sqrt(discriminant)) / (2 * a);
            else
                delta = 2 * c / (-b + sqrt(discriminant));
            end

            % Take the step but make sure we stay in the region.
            if delta < xj
                xj = xj - delta;
            else
                xj = .25 * xj;
            end

        end

    end


end
