function [lambda,V] = solve2x2( alpha, beta, gamma, firsts_positive )
%
%   [lambda,V] = solve2x2( alpha, beta, gamma )
%
%   Computes the spectral factorization of a real symmetric 2x2 matrix A =
%   [alpha beta;beta gamma] with non-negative off-diagonal beta. This uses
%   a method adapted from [1] restricted to the case where beta >= 0. It
%   returns the eigenvalues in ascending order. The optioinal logical
%   argument firsts_positive defaults to true if not supplied by the user.
%   If true the signs of the first row of V are positive, if false then the
%   elements of the second row of V will all be positive.
%
%   [1] C. F. Borges, An improved formula for Jacobi rotations, arXiv:
%   1806.07876v1, 2018.
%
%   10 MARCH 2017 by Carlos F. Borges

% Argument validation.
arguments
    alpha (1,1) {mustBeReal}
    beta (1,1) {mustBeNonnegative}
    gamma (1,1) {mustBeReal}
    firsts_positive logical = true
end

% This is a trivial but rare case. We put it up front and expect branch
% prediction to toss it.
if alpha == gamma 
    lambda = [alpha-beta;alpha+beta];
    if firsts_positive
        V = [1 1;-1 1]*sqrt(.5);
    else
        V = [-1 1;1 1]*sqrt(.5);
    end
    return;
end

delta = .5*(alpha-gamma);
if delta > 0
    sigma = beta/(hypot(delta,beta) + delta); % Partial computation of sigma.
    s = 1/sqrt(1+sigma*sigma); c = sigma*s; % Construct c and s.
    sigma = beta*sigma; % Finish computation of sigma.
    lambda = [gamma-sigma; alpha+sigma];
else
    sigma = beta/(hypot(delta,beta) - delta); % Partial computation of sigma.
    c = 1/sqrt(1+sigma*sigma); s = sigma*c; % Construct c and s.
    sigma = beta*sigma; % Finish computation of sigma.
    lambda = [alpha-sigma; gamma+sigma];
end

if firsts_positive
    V = [c s; -s c ];
else
    V = [-c s; s c ];
end
    
end