function [lambda, u_1, u_n] = tqrPSF(alpha, beta, firsts_positive)
%
%   [lambda, u_1, u_n] = tqrPSF(alpha, beta, firsts_positive)
%
%   Inputs: 
%       alpha - n diagonal elements, all real
%       beta - n-1 sub(sup)-diagonal elements, all real
%       firsts_positive - optional argument that controls the signs of u_k
%
%   Outputs:
%       lam - eigenvalues ordered from left to right
%       u_1 - first elements of the normalized eigenvectors
%       u_n - last elements of the normalized eigenvectors (optional)
%
%   Solves the symmetric tridiagonal eigenproblem using a version of the
%   (real symmetric) TQR algorithm with the Wilkinson shift that appears in
%   [1]. In practice this is an O(n^2) process. The computed eigenvalues
%   (lambda) are real and the output is sorted so that the eigenvalues are
%   increasing. The output column vectors u_1 and u_n contain the elements
%   of the first and last rows, respectively, of the matrix of normalized
%   eigenvectors. If there are only two output arguments then the code
%   becomes a standard TQR Golub-Welsch code that computes and returns only
%   the eigenvalues and the positive first elements of the normalized
%   eigenvectors. The logical argument firsts_positive is optional and
%   defaults to true if not supplied by the user. If true the signs of the
%   elements of u_1 and u_n are permuted so that all of the elements of u_1
%   are positive, if false then the elements of u_n will all be positive.
%   The input alpha must have exactly on more element than beta.
%
%   [1] K.Gates, W. B. Gragg, Notes on TQR algorithms, Journal of
%   Computational and Applied Mathematics, 86 (1997), pp. 195{203,

%   23 JAN 2024 by Carlos F. Borges.

% Argument validation
arguments
    alpha(:, 1) {mustBeReal}
    beta(:, 1) {mustBeReal}
    firsts_positive logical = true
end
if length(alpha) ~= (length(beta)+1)
    error('Inputs alpha and beta are incompatibly sized.')
end

% It will be cleaner to proceed if we take all of the betas to be
% non-negative. This is easy to arrange for this calculation as we want
% only the eigenvalues and the first and last rows of the eigenvector
% matrix. The eigenvalues clearly do not depend on the signs of the betas
% but the sign relationship between u_1 and u_n will change if there are an
% odd number of negative betas. So we set flip_sign to true in that case
% so that it can be corrected later on and immediately replace beta with
% abs(beta).
flip_sign = (mod(sum(beta<0),2) > 0);

% Initialize
eps2 = .25*eps*eps; % The square of the unit roundoff.
n = length(alpha);
MAX_ITER = 10 * n;

% Pad beta with a leading zero and eliminate the signs to simplify the
% code.
beta = [0; abs(beta(:))];

% Initialize the U matrix that is used to accumulate the rotations applied
% to the appropriate axis vectors.
if nargout > 2
    U = zeros(n,2); U(1,1) = 1; U(end,2) = 1;
else
    U = zeros(n,1); U(1) = 1;
end

% Main loop - Finds the eigenvalues one at a time.
for m = n:-1:2

    % Do tqr iterations to compute the mth eigenvalue.
    for iter = 0:MAX_ITER

        % Test for deflation using criterion from LAPACK.
        if beta(m)*beta(m) <= eps2*abs(alpha(m-1)*alpha(m))
            beta(m) = 0;
            break
        else

            % Compute the Wilkinson shift as a perturbation of the
            % Rayleigh shift r = alpha(m).
            delta = .5*(alpha(m-1)-alpha(m));
            % This would be faster with copysign() if it were available.
            if delta >= 0
                sigma = beta(m)*(beta(m)/(delta + hypot(delta,beta(m))));
            else
                sigma = beta(m)*(beta(m)/(delta - hypot(delta,beta(m))));
            end
            wilkinson_shift = alpha(m)-sigma;

            % Take a step of the tqr algorithm.  
            c = 1;
            s = 0;
            p = wilkinson_shift - alpha(1);
            t = p;

            % There are many ways to implement the inner loop.  The
            % algorithmic form given here uses one comparison and one
            % hypot() call per step plus either 17 flops, for a
            % Golub-Welsch construction, or 23 flops, for a PSF
            % construction.
            for j = 1:m - 1

                %  Compute the two by two reflector carefully and update
                %  beta(j).
                oldc = c;
                oldt = t;

                h = hypot(p,beta(j+1));
                beta(j) = h*s;
                c = p/h;
                s = beta(j+1)/h;

                % Update p, t, alpha(j), and U(j:j+1,:).
                p = c * (wilkinson_shift - alpha(j+1)) - s * beta(j+1) * oldc;
                t = c * p;
                alpha(j) = alpha(j+1) + t - oldt;
                U(j:j + 1,:) = [-c, s; s, c]*U(j:j + 1,:);

            end

            % Update alpha(m), beta(m), and U(m,:).
            alpha(m) = wilkinson_shift - t;
            if p >= 0
                beta(m) = p * s;
                U(m,:) = -U(m,:);
            else
                beta(m) = -p * s;
            end

        end

    end

    % If the code reached the iteration limit then there was a problem.
    if iter == MAX_ITER
        error('The tqrPSF iteration did not terminate within the limit')
    end

end

% Prepare the output.
[lambda, p] = sort(alpha);
U = normc(U(p,:));

% If this is a Golub-Welsch call then just return the positive first
% elements.
if nargout < 3
    u_1 = abs(U);
    return;
end

% For a PSF call we need to do a bit more work to get the signs all right.
if firsts_positive
    sign_swaps = (U(:,1) < 0);
else
    sign_swaps = (U(:,2) < 0);
end
U(sign_swaps,:) = -U(sign_swaps,:);
u_1 = U(:,1);
u_n = U(:,2);

% Fix the sign disagreement if flip_sign
if flip_sign
    if firsts_positive
        u_n = -u_n;
    else
        u_1 = - u_1;
    end
end

% Allow the user to collect a consolidated output.
if nargout < 2
    lambda = [lambda u_1 u_n];
end

end % end tqr